using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;

namespace PropertyNamesGenerator.CodeGenerator
{
    public abstract class CodeFileGenerator
    {
        private TextWriter _writer;
        private string _fileName;
        private readonly string _className;

        protected CodeFileGenerator(string className, string fileName)
        {
            _className = className;
            _fileName = fileName;
        }

        #region Public 

        public void CreateFile()
        {
            FileStream file = File.Create(FileName);
            file.Close();
        }

        public virtual void FillContents(List<string> properties)
        {
            string generatedClassName = string.Format("PropertyNames{0}Class", _className);

            WriteHeader();
            WriteNamespace();
            CreateAccessClass(_className, generatedClassName);
            CreateInternalClass(generatedClassName, properties);
            FinalizeGeneration();
        }

        public string FileName
        {
            get
            {
                if (!FileHasExtension(_fileName))
                    _fileName = _fileName + "." + GetFileExtension();
                return _fileName;
            }
        }

        #endregion

        #region File extension


        private static bool FileHasExtension(string fileName)
        {
            return
                (
                    (fileName.Contains("\\") && (fileName.LastIndexOf("\\") < fileName.LastIndexOf(".")))
                     || (!fileName.Contains("\\") && fileName.Contains("."))
                );
        }

        protected abstract string GetFileExtension();


        #endregion

        #region Contents

        

        protected TextWriter Writer
        {
            get
            {
                if (_writer == null)
                {
                    _writer = new StreamWriter(FileName); ;
                }
                return _writer;
            }
        }

        public abstract void WriteHeader();

        protected void WriteHeader(string commentCharacter)
        {
            Writer.WriteLine(string.Format("{0}***************************************************************", commentCharacter));
            Writer.WriteLine(string.Format("{0}      This file was generated by PropertyNamesGenerator 0.1.0.0", commentCharacter));
            Writer.WriteLine(string.Format("{0}     Any change to this file will be lost if regenerated.", commentCharacter));
            Writer.WriteLine(string.Format("{0}***************************************************************", commentCharacter));
            Writer.WriteLine();
        }

        public abstract void WriteNamespace();
        public abstract void CreateAccessClass(string className, string generatedClassName);
        public abstract void CreateInternalClass(string generatedClassName, List<string> properties);
        
        public virtual void FinalizeGeneration()
        {
            Writer.Close();
        }

        #endregion
    }
}
