using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using PropertyNamesGenerator.CodeGenerator;
using PropertyNamesGenerator.Parsers;
using PropertyNamesGenerator.Util;

namespace PropertyNamesGenerator
{
    public class Generator
    {

        public static void GeneratePropertyNameClasses(string fileString, string searchString, string outputDir, Language language)
        {
            List<ClassDictionary> listOfClassHierarchies = new List<ClassDictionary>();
            
            DirectoryUtil.SetCurrentDirectory(fileString);

            Output.NotifyUser("Getting files from '" + Directory.GetCurrentDirectory()  + "', with searchstring:" + searchString, OutputLevel.Info);
            string[] files = Directory.GetFiles(Directory.GetCurrentDirectory(), searchString);

            Output.NotifyUser("Parsing " + files.Length + " files", OutputLevel.Info);
            foreach (string file in files)
            {
                if (file.Contains("hbm.xml"))
                {
                    Output.NotifyUser("Found hbm-file: " + file, OutputLevel.Info);
                    listOfClassHierarchies.Add(HbmRegexParser.ParseDocument(file));
                }
                else
                {
                    Output.NotifyUser(string.Format("No known way of parsing this file: {0}", file), OutputLevel.Error, ConsoleColor.Red);
                }
            }

            GenerateClasses(listOfClassHierarchies, outputDir, language);

        }

        private static void GenerateClasses(List<ClassDictionary> input, string outputDir, Language language)
        {
            DirectoryUtil.SetCurrentDirectory(outputDir);

            Output.NotifyUser(string.Format("Looping through all {0} identified class hierarchies", input.Count), OutputLevel.Info);

            foreach (ClassDictionary classList in input)
            {
                foreach (KeyValuePair<string, List<string>> pair in classList)
                {
                    GenerateClass(pair.Key, pair.Value, outputDir, language);
                }
            }
        }

        private static void GenerateClass(string name, List<string> properties, string outputDir, Language language)
        {
            string createFileName = Directory.GetCurrentDirectory() + "\\" + name; 
            Output.NotifyUser("Generating class " + createFileName, OutputLevel.Info);

            CodeFileGenerator generator = CodeFileGeneratorFactory.CreateGenerator(name, createFileName, language);
            generator.CreateFile();
            generator.FillContents(properties);
            
            Output.NotifyUser(string.Format("Successfully generated file {0}", generator.FileName), OutputLevel.Always, ConsoleColor.Yellow);
        }
    }
}
