using System;
using System.Collections.Generic;
using System.Text;

namespace PropertyNamesGenerator.Util
{
    internal enum OutputLevel
    {
        Info = 1,
        Error = 2,
        Always = 3
    }

    /// <summary>
    /// 
    /// </summary>
    internal static class Output
    {
        private static OutputLevel _trigger = OutputLevel.Error;

        internal static void SetOutputLevelTrigger(OutputLevel level)
        {
            _trigger = level;
        }

        internal static void NotifyUser(string message, OutputLevel priority)
        {
            NotifyUser(message, priority, Console.ForegroundColor);
        }

        internal static void NotifyUser(string message, OutputLevel priority, ConsoleColor color)
        {
            if (_trigger == OutputLevel.Error && priority < OutputLevel.Error)
                return;

            ConsoleColor previousColor = Console.ForegroundColor;
            Console.ForegroundColor = color;

            Console.WriteLine(message);
            Console.ForegroundColor = previousColor;
            
        }

        internal static void WriteHelpMessage()
        {
            Output.NotifyUser("Run like: PropertyNamesGenerator.exe /files:\"path/*.hbm.xml\" /out:\"path/PropertyNames/\" /lang:cs", OutputLevel.Always);
            Output.NotifyUser("Parameters:", OutputLevel.Always);
            Output.NotifyUser("/Files [Mandatory] - Must have both a directory and a search string.", OutputLevel.Always);
            Output.NotifyUser("/Out [Mandatory] - Must specify an output directory.", OutputLevel.Always);
            Output.NotifyUser("/lang [Optional] - Output-language. C# or VB. (C# is default)", OutputLevel.Always);
            Output.NotifyUser("-V - Verbose output.", OutputLevel.Always);
            Output.NotifyUser("Note: All directories can be relative.", OutputLevel.Always);
        }

    }
}